/**
 * Copyright (c) 2006 - 2008 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.uv.client.rtmp.INetConnection;
import com.smaxe.uv.client.rtmp.ISharedObject;

import java.util.List;
import java.util.Map;

/**
 * <code>ExRtmpConnectRemoteSharedObject</code> - {@link NetConnection} usage example.
 * <p> Note:
 * <br> - The example shows how to connect to the remote shared object, set shared object values.
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmp-client" target="_blank">JUV RTMP Client</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmfp-client" target="_blank">JUV RTMFP/RTMP Client</a>
 */
public final class ExRtmpConnectRemoteSharedObject extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        
        // Android-specific:
        // - please add permission to the AndroidManifest.xml
        // <uses-permission android:name="android.permission.INTERNET" />
        // - please use separate thread to connect to the server (not UI thread) : 
        // NetConnection#connect() connects to the remote server in the invocation thread,
        // so it causes NetworkOnMainThreadException on 4.0 (http://developer.android.com/reference/android/os/NetworkOnMainThreadException.html)
        com.smaxe.uv.client.rtmp.License.setKey("SET-YOUR-KEY");
        
        final ExRtmpConnectRemoteSharedObject app = new ExRtmpConnectRemoteSharedObject();
        
        // RTMP example
        app.start(new com.smaxe.uv.client.rtmp.NetConnection(), "rtmp://localhost:1935/live");
        
        // RTMFP example
        // Note: This classes are available in the JUV RTMFP/RTMP Client library (juv-rtmfp-client-*.jar)
        // com.smaxe.uv.client.rtmfp.License.setKey("SET-YOUR-KEY");
        // 
        // app.start(new com.smaxe.uv.client.rtmfp.NetConnection(), "rtmfp://localhost:1935/live");
    }
    
    
    /**
     * <code>SharedObjectAPI</code> - SharedObject API : SharedObject invokes methods of this class.
     */
    public static class SharedObjectAPI extends Object
    {
        /**
         * Constructor.
         */
        public SharedObjectAPI()
        {
        }
        
        /**
         * Test shared object.
         * 
         * @param parameter
         */
        public void testSharedObject(final String parameter)
        {
            System.out.println("SharedObjectAPI#testSharedObject(): " + parameter);
        }
    }
    
    /**
     * <code>NetConnectionListener</code> - {@link NetConnection} listener implementation.
     */
    public class NetConnectionListener extends INetConnection.ListenerAdapter
    {
        /**
         * Constructor.
         */
        public NetConnectionListener()
        {
        }
        
        @Override
        public void onAsyncError(final INetConnection source, final String message, final Exception e)
        {
            System.out.println("NetConnection#onAsyncError: " + message + " " + e);
        }
        
        @Override
        public void onIOError(final INetConnection source, final String message)
        {
            System.out.println("NetConnection#onIOError: " + message);
        }
        
        @Override
        public void onNetStatus(final INetConnection source, final Map<String, Object> info)
        {
            System.out.println("NetConnection#onNetStatus: " + info);
            
            final Object code = info.get("code");
            
            if (INetConnection.CONNECT_SUCCESS.equals(code))
            {
            }
            else
            {
                disconnected = true;
            }
        }
    }
    
    
    // fields
    private volatile boolean disconnected = false;
    
    /**
     * Constructor.
     */
    public ExRtmpConnectRemoteSharedObject()
    {
    }
    
    /**
     * Starts the example.
     * 
     * @param connection
     * @param url
     * @throws Exception if an exception occurred
     */
    public void start(final INetConnection connection, final String url) throws Exception
    {
        connection.addEventListener(new NetConnectionListener());
        
        connection.connect(url);
        
        // wait till connected
        while (!connection.connected() && !disconnected)
        {
            try
            {
                Thread.sleep(100);
            }
            catch (Exception e) {/*ignore*/}
        }
        
        if (!disconnected)
        {
            ISharedObject so = connection.createSharedObject("mySharedObject", false /*persistent*/);
            
            so.addEventListener(new ISharedObject.ListenerAdapter()
            {
                @Override
                public void onNetStatus(ISharedObject source, final Map<String, Object> info)
                {
                    System.out.println("SharedObject#onNetStatus: " + info);
                }
                
                @Override
                public void onSync(ISharedObject source, final List<ISharedObject.Change> changeList)
                {
                    for (ISharedObject.Change change : changeList)
                    {
                        switch (change.code)
                        {
                            case ISharedObject.Change.CONNECT:
                            {
                                System.out.println("SharedObject " + source.getName() + " : " + "connected");
                                
                                source.data().put("attr1", 123);
                                
                            } break;
                            case ISharedObject.Change.CHANGE:
                            {
                                System.out.println("SharedObject " + source.getName() + " : " +
                                        "attribute " + change.attribute + " changed from " + change.oldValue + " to " + change.newValue);
                            } break;
                            case ISharedObject.Change.STATUS:
                            {
                                System.out.println("SharedObject " + source.getName() + " : " +
                                        "status " + change.attribute);
                            } break;
                        }
                    }
                }
            });
            
            so.client(new SharedObjectAPI());
            
            so.connect(connection, "parameter");
            
            Thread.sleep(5 * 1000);
            
            so.send("testSharedObject", "Hello, SharedObject!");
            
            Thread.sleep(5 * 1000);
            
            so.close();
        }
        
        connection.close();
    }
}